@@ -3,47 +3,59 @@
     elements, and UI elements to forms generated via the uni_form template tag.
 
 """
-from uni_form.util import BaseInput, Toggle
-from django.utils.safestring import mark_safe
-from django.template.loader import render_to_string
+from django.core.urlresolvers import reverse, NoReverseMatch
 from django.forms.forms import BoundField
+from django.template.loader import render_to_string
+from django.utils.safestring import mark_safe
+
+
+from uni_form.util import BaseInput, Toggle
+
+
+class FormHelpersException(Exception):
+    """ This is raised when building a form via helpers throws an error.
+        We want to catch form helper errors as soon as possible because
+        debugging templatetags is never fun.
+    """
+    pass
+
 
 class Submit(BaseInput):
     """
-        Used to create a Submit button descriptor for the uni_form template tag.    
+        Used to create a Submit button descriptor for the uni_form template tag.
     """
-
-    input_type = 'submit'    
+    
+    input_type = 'submit'
 
 
 class Button(BaseInput):
     """
-        Used to create a Submit input descriptor for the uni_form template tag.    
+        Used to create a Submit input descriptor for the uni_form template tag.
     """
     
-    input_type = 'button'    
+    input_type = 'button'
 
 
 class Hidden(BaseInput):
     """
-        Used to create a Hidden input descriptor for the uni_form template tag.    
+        Used to create a Hidden input descriptor for the uni_form template tag.
     """
-
-    input_type = 'hidden'  
     
+    input_type = 'hidden'
+
 class Reset(BaseInput):
     """
-        Used to create a Hidden input descriptor for the uni_form template tag.    
+        Used to create a Hidden input descriptor for the uni_form template tag.
     """
-
-    input_type = 'reset'      
+    
+    input_type = 'reset'
 
 def render_field(field, form):
     if isinstance(field, str):
         return render_form_field(form, field)
     else:
         return field.render(form)
-            
+
 def render_form_field(form, field):
     try:
         field_instance = form.fields[field]
@@ -77,7 +89,7 @@ class Layout(object):
     '''
     def __init__(self, *fields):
         self.fields = fields
-        
+    
     def render(self, form):
         html = ""
         for field in self.fields:
@@ -88,9 +100,9 @@ class Layout(object):
         return html
 
 class Fieldset(object):
-
+    
     ''' Fieldset container. Renders to a <fieldset>. '''
-
+    
     def __init__(self, legend, *fields, **args):
         if 'css_class' in args.keys():
             self.css = args['css_class']
@@ -98,7 +110,7 @@ class Fieldset(object):
             self.css = None
         self.legend_html = legend and ('<legend>%s</legend><hr/>' % unicode(legend)) or ''
         self.fields = fields
-        
+    
     
     def render(self, form):
         if self.css:
@@ -110,7 +122,7 @@ class Fieldset(object):
             html += render_field(field, form)
         html += u'</fieldset>'
         return html
-    
+
 
 
 class Row(object):
@@ -121,14 +133,14 @@ class Row(object):
             self.css = kwargs['css_class']
         else:
             self.css = "formRow"
-            
+    
     def render(self, form):
         output = u'<div class="%s">' % self.css
         for field in self.fields:
             output += render_field(field, form)
         output += u'</div>'
         return u''.join(output)
-    
+
 class Column(object):
     ''' column container. Renders to a set of <div>'''
     def __init__(self, *fields, **kwargs):
@@ -137,21 +149,21 @@ class Column(object):
             self.css = kwargs['css_class']
         else:
             self.css = "formColumn"
-            
+    
     def render(self, form):
         output = u'<div class="%s">' % self.css
         for field in self.fields:
             output += render_field(field, form)
         output += u'</div>'
         return u''.join(output)
-    
+
 class HTML(object):
-
+    
     ''' HTML container '''
-
+    
     def __init__(self, html):
         self.html = unicode(html)
-
+    
     def render(self, form):
         return self.html
 
@@ -160,8 +172,8 @@ class HTML(object):
 
 class FormHelper(object):
     """
-        By setting attributes to me you can easily create the text that goes 
-        into the uni_form template tag. One use case is to add to your form 
+        By setting attributes to me you can easily create the text that goes
+        into the uni_form template tag. One use case is to add to your form
         class.
         
         First we create a MyForm class and instantiate it
@@ -177,42 +189,76 @@ class FormHelper(object):
         ...     helper.form_class = 'search'
         ...     submit = Submit('search','search this site')
         ...     helper.add_input(submit)
-        ...     reset = Reset('reset','reset button')                
-        ...     helper.add_input(reset)        
+        ...     reset = Reset('reset','reset button')
+        ...     helper.add_input(reset)
         
         After this in the template:
-        
+            
             {% load uni_form %}
             {% uni_form form form.helper %}
-            
         
+    
     """
     
     def __init__(self):
+        self._form_method = 'POST'
+        self._form_action = ''
         self.form_id = ''
         self.form_class = ''
         self.inputs = []
         self.toggle = Toggle()
         self.layout = None
+    
+    def get_form_method(self):
+        return self._form_method
+    
+    def set_form_method(self, method):
+        if method.lower() not in ('get','post'):
+            raise FormHelpersException('Only GET and POST are valid in the \
+                    form_method helper attribute')
         
+        self._form_method = method.upper()
+    
+    # we set properties the old way because we want to support pre-2.6 python
+    form_method = property(get_form_method, set_form_method)
+    
+    def get_form_action(self):
+        return self._form_action
+    
+    def set_form_action(self, action):
+        try:
+            self._form_action = reverse(action)
+        except NoReverseMatch, e:
+            msg = 'Your form action needs to be a named url defined in a urlconf file\n'
+            msg += 'Your broken action is: %s\n' % action
+            msg += 'NoReverseMatch: %s' % e
+            raise FormHelpersException(msg)
+    
+    # we set properties the old way because we want to support pre-2.6 python
+    form_action = property(get_form_action, set_form_action)
+    
     def add_input(self, input_object):
         self.inputs.append(input_object)
-        
+    
     def add_layout(self, layout):
         self.layout = layout
-        
+    
     def render_layout(self, form):
         return mark_safe(self.layout.render(form))
-        
+    
     def get_attr(self):
         items = {}
+        items['form_method'] = self.form_method.strip()
+        
+        if self.form_action:
+            items['form_action'] = self.form_action.strip()
         if self.form_id:
-            items['id'] = self.form_id.strip()   
+            items['id'] = self.form_id.strip()
         if self.form_class:
             items['class'] = self.form_class.strip()
         if self.inputs:
             items['inputs'] = self.inputs
         if self.toggle.fields:
             items['toggle_fields'] = self.toggle.fields
-        return items    
+        return items
         
